@@ -11,15 +11,21 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <db.h>
 #include <dirent.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 
-#define DOCDIR "/usr/doc"           
-#define INDEXROOT "/usr/doc/HTML"
+#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 1
+  #include <db1/db.h>
+#else
+  #include <db.h>
+#endif
+
+
+#define DOCDIR "/usr/share/doc"           
+#define INDEXROOT "/usr/share/doc/HTML"
 #define DBNAME "/var/lib/dhelp/dbase"
 #define DBTITLE_NAME "/var/lib/dhelp/titles"
 
@@ -148,7 +154,12 @@ DB *db_title_open (int flags)
 {
   HASHINFO hash;
 
-  hash.bsize = 256;
+  #ifdef __alpha__       /* chris@beezer.med.miami.edu */
+    hash.bsize = 512;
+  #else
+    hash.bsize = 256;
+  #endif
+
   hash.ffactor = 8;
   hash.nelem = 1;
   hash.cachesize = 5000;
@@ -324,7 +335,7 @@ void html_w_head (FILE *out, DB *tdb)
   strcat (zw, "dhelp/debian.jpg");
   fprintf (out, "<IMG SRC=\"%s\" ALT=\"Debian GNU/Linux\"> \n", zw);
   fprintf (out, "<H1>%s</H1>\n<P>\n", zw2);
-  fprintf (out, "<TABLE>\n");
+  fprintf (out, "<DL>\n");
 
   html_error (out);
 }
@@ -356,7 +367,7 @@ void html_w_tail (FILE *out)
   if (i > lang)
     i = 0;         
 
-  fprintf (out, "</TABLE>\n");
+  fprintf (out, "</DL>\n");
   fprintf (out, "<P>\n<CENTER>\n"
                 "<FORM ACTION=\"/cgi-bin/dsearch\">\n"
                 "<INPUT TYPE=\"text\" NAME=\"search\">\n"
@@ -379,8 +390,10 @@ void html_w_item (tItem *item, FILE *out)
   clearerr (out);
 
   path_relative (zw);
-  fprintf (out, "<TR VALIGN=top><TD><IMG SRC=\"%sdhelp/text.gif\">"
-                "<TD><A HREF=\"%s%s\">%s</A><TD>%s\n",
+  fprintf (out, "<DT><IMG SRC=\"%sdhelp/text.gif\">"
+                " <A HREF=\"%s%s\"><FONT FACE=\"Helvetica, Arial\">"
+                "<B>%s</B></FONT></A><P></DT>\n"
+                "    <DD>%s</DD>\n\n",
                 zw, zw, item->file, item->name, item->descrip);
   
   html_error (out);
@@ -474,8 +487,11 @@ int html_w_dir_links (char *dirname, DB *tdb)
       if (S_ISDIR (buf.st_mode))
       {
         path_relative (relpath);
-        fprintf (out, "<TR VALIGN=top><TD><IMG SRC=\"%sdhelp/folder.gif\">\n"
-                      "               <TD><A HREF=\"%s/index.html\">%s/</A>\n",
+        fprintf (out, "<DT><IMG SRC=\"%sdhelp/folder.gif\">"
+                 " <A HREF=\"%s/index.html\">"
+                 "<FONT FACE=\"Helvetica, Arial\"><B>%s/</B></FONT>"
+                 "</A></DT>\n"
+                 "    <DD></DD>\n\n",
                  relpath, dir_entry->d_name, dir_entry->d_name);
         if (html_w_dir_links (zw, tdb))
 	{
@@ -498,22 +514,32 @@ void html_write (void)
 {
   tItem item;
   DB    *db, *tdb;
-  int i;
+  int   i;
+  FILE  *out;
 
   if ((db = db_open (O_RDONLY)) == NULL)
   {
     fprintf (stderr, "dhelp_parse: can't open %s\n\n", DBNAME);
-    exit (1);
+    exit (0);
   }
-  if ((tdb = db_title_open (O_RDONLY)) == NULL)
+  if ((tdb = db_title_open (O_RDWR|O_CREAT)) == NULL)
   {
     fprintf (stderr, "dhelp_parse: can't open %s\n\n", DBTITLE_NAME);
-    exit (1);
+    exit (0);
   }
 
   rm_r (INDEXROOT);
   mkdir (INDEXROOT, 0755);
   chdir (INDEXROOT);        /* usr/doc/HTML */
+    
+  if ((out = fopen ("README", "w")) != NULL)
+  {
+    fprintf (out, "\nDon't put files in this directory!\n"
+                  "dhelp will delete *all* files in this directory "
+                  "when creating a new index.\n\n"
+                  "Marco Budde (budde@debian.org)\n\n");
+    fclose (out);
+  }
   
   db_read (db, &item);
   do
@@ -661,12 +687,13 @@ int dhelp_del (char *name)
   char  *file_ptr;
   char  resolv_docdir[MAXPATHLEN];
 
+
   if ((in = fopen (name, "r")) == NULL)
   {
     fprintf (stderr, "dhelp_parse: can't open %s\n\n", name);
     return (1);
   }
-  if ((db = db_open (O_RDWR)) == NULL)
+  if ((db = db_open (O_RDWR|O_CREAT)) == NULL)
   {
     fprintf (stderr, "dhelp_parse: can't open %s\n\n", DBNAME);
     return (1);
@@ -750,6 +777,8 @@ int main (int argc, char *argv[])
     dhelp_add_rec (resolv_name);
   }
   else
+  {
+    mkdir ("/var/lib/dhelp/", 0755);
     for (i=2; i < argc; i++)
     { 
       if (!strncmp (argv[i], DOCDIR, strlen (DOCDIR)))
@@ -766,6 +795,12 @@ int main (int argc, char *argv[])
           default:  help ();
         }  
       }
+      else if (!strncmp (argv[i], "/usr/doc", 8))
+      { 
+        fprintf (stderr, "dhelp_parse: Warning - documents in /usr/doc "
+                 "are no longer supported!\n\n");
+        return (0);   /* do not break the installation of old packages */
+      }
       else
       {
         fprintf (stderr, "dhelp_parse: You can add only directories "
@@ -773,6 +808,7 @@ int main (int argc, char *argv[])
         return (1);
       } 
     } 
+  }
 
   html_write ();
    
